function CODIGO_PORTICOS
% Anlisis Lineal
% Teora de Mecanismos (TM)
% Carga distribuida (p) y puntual (F)

% Datos
% Caractersticas geomtricas
L=1.0;
% Caractersticas de la seccin
Iz=8360.0*10^-8;
S=628.0*10^-6;
% Caractersticas del material
vE=2.1*10^11;
sigma_F=275*10^6;
Mp=S*sigma_F;
% Cargas
p=0;
F=6000;
% Geometra 
nodos = {{[0, 0], {'Fx', 'Fy', 'Mz'}}, % empotramiento   
    {[0, 2*L], { 0 ,0, 0}},
    {[2*L, 2*L], {0, -F, 0}},
    {[4*L, 2*L], {F, 0, 0}},
    {[4*L,0], {'Fx', 'Fy', 'Mz'}} % apoyo fijo
    }; % coordenadas / fuerzas
material = {{vE}}; % una lista por material
perfil = {{Iz, Mp}}; % una lista por tipo de perfil
barras = {{[1, 2], [1, 1], 0},
    {[2, 3], [1, 1], 0},
    {[3, 4], [1, 1], 0},    
    {[4, 5], [1, 1], 0}, 
    }; % definicion, propiedades(material,perfil)

nd=length(nodos);
nb=length(barras);
% numero de ecuaciones de equilibrio
NPR=nd;
R=0;
for i=1:nd
    kk1=nodos{i}{2}{1}; 
    kk2=nodos{i}{2}{2};
    kk3=nodos{i}{2}{3};    
    if (kk1=='Fx')&(kk2=='Fy')&(kk3=='Mz')
        R=R+3;  % empotramiento     
    elseif (kk1=='Fx')&(kk2=='Fy')&(kk3==0)
        R=R+2;    %apoyo fijo
        NPR=NPR-1;
    elseif (kk1=='Fx')&(kk3==0)
        R=R+1;    %apoyo mvil eje x
        NPR=NPR-1;
    elseif (kk2=='Fy')&(kk3==0)
        R=R+1;    %apoyo mvil eje y
        NPR=NPR-1;
    end     
end

GH=(3*nb+R)-(3*nd);
EQ=NPR-GH;

fprintf('Ecuaciones de equilibrio = %d \n',EQ);
fprintf('Ecuaciones de compatibilidad = %d \n',GH);

% Giros y desplazamientos virtuales de los nodos
dv=0.0001;
mt=zeros(nd,3,3); % matrices Ti
mmT=zeros(3,3); % matriz T
qi=zeros(EQ,nd); % giros virtuales
di=zeros(EQ,2,nd); % desplazamientos virtuales
Mi=ones(1,nd); % momentos reales
% mecanismo (MC1)
q0=[0 1 1 1 0];
options = optimoptions('fsolve','Display','iter','TolFun',1e-30,'TolX',1e-50);
[q,fun]=fsolve(@posicion,dv*q0,options);
qi(1,:)=q;
% desplazamiento de los nodos
di(1,:,:)=delta(q);
% mecanismo (MC2)
q0=[1 1 0 1 1];
[q,fun]=fsolve(@posicion,dv*q0,options);
qi(2,:)=q;
di(2,:,:)=delta(q);

% Momentos virtuales
mi=zeros(GH,nd); % momentos virtuales
m0=[1 0 0 1 0]; % EC1
[m,fun]=fsolve(@momentosVirtuales,m0,options);
mi(1,:)=m;
m0=[1 0 1 0 0]; % EC2
[m,fun]=fsolve(@momentosVirtuales,m0,options);
mi(2,:)=m;
m0=[0 0 1 0 0]; % EC3
[m,fun]=fsolve(@momentosVirtuales,m0,options);
mi(3,:)=m;
% Resultados 
s0=zeros(1,EQ+GH);
options = optimoptions('fsolve','Display','iter','TolFun',1e-30,'TolX',1e-50);
[s,fval]=fsolve(@ecuaciones,s0,options);
fprintf('Momento flector en "a": %5.2f \n',s(1));
fprintf('Momento flector en "b": %5.2f \n',s(2));
fprintf('Momento flector en "c": %5.2f \n',s(3));
fprintf('Momento flector en "d": %5.2f \n',s(4));
fprintf('Momento flector en "e": %5.2f \n',s(5));
        
    function v=posicion(q)
       q=q.*q0; 
       kk=matrizT(q);
       vP=cat(1,kk(1:2,3),kk(2,1));
       v0=cat(2,nodos{nd}{1},0.0);
       v=v0'-vP;
    end


    function v=matrizT(q)

        lx=0;
        ly=0;
        i=1;
        Li=norm(nodos{i+1}{1}-nodos{i}{1});        
        v=mT(q(1),Li,lx,ly);
        mt(1,:,:)=v;

        for i=1:nb            
            Li=norm(nodos{i+1}{1}-nodos{i}{1});
            kk=(nodos{i+1}{1}-nodos{i}{1})/Li;
            lx=kk(1);
            ly=kk(2); 
            kk=mT(q(i+1),Li,lx,ly);
            mt(i+1,:,:)=kk;
            v=v*kk;   

        end

        mmT=v;

        function v=mT(x,Li,lx,ly)            
            v=[[1,-x,Li*lx]
                [x,1,Li*ly]
                [0,0,1]
                ];                             
        end

    end

    function v=delta(q)
        v=zeros(2,nd);   
        for i=1:nd
            kk1=diag(ones(1,3));
            for j=1:i               
                kk1=kk1*reshape(mt(j,:,:),[3,3]);                
            end
            kk=kk1(1:2,3);
            kk2=nodos{i}{1}';
            v(:,i)=kk-kk2;
        end                            
    end  

    function v=momentosVirtuales(m)
        v=zeros(1,EQ);
        q0=qi;
        for j=1:EQ      
            q0(j,:)=q0(j,:)/max(q0(j,:));
            for i=1:nd      
                v(j)=v(j)-m*(q0(j,:)');     
            end
        end                         
    end

    function v=ecuaciones(s)
        M=s(:); % momentos reales                     
        v=zeros(1,EQ+GH);
        % aplicar el Principio de los Desplazamientos Virtuales (PDV)        
        v(1:EQ)=PDV(di); % ecuaciones de equilibrio
        % aplicar PFV sistematico
        v(EQ+1:end)=PFV(mi); % GH ecuaciones de compatibilidad 
        function v=PDV(di) % aplica el PDV -> obtener EQ ecuaciones de equilibrio             
            v=zeros(1,EQ);
            for j=1:EQ
                desp=zeros(2,nd);
                desp=reshape(di(j,:,:),[2,nd]);
                for i=1:nd % nudos
                    v(j)=v(j)-M(i)*qi(j,i);    
                    kk=nodos{i}{2};
                    if kk{1}~='Fx'            
                     v(j)=v(j)+(desp(1,i)*kk{1}+desp(2,i)*kk{2});
                    end
                end
                for i=1:nb % barras
                    ij=barras{i}{1};
                    Li=norm(nodos{i+1}{1}-nodos{i}{1});                                  
                    p0=barras{i}{3};
                    a=desp(1,ij(1));
                    b=(desp(1,ij(2))-desp(1,ij(1)))/Li;
                    d=Li*(a+b*Li/2); % integral del desplazamiento
                    v(j)=v(j)+p0*d;
                end                    
            end
        end
        function v=PFV(mi) % aplica el PFV -> obtener GH ecuaciones de compatibilidad
            v=zeros(1,GH);
            for j=1:GH
                for i=1:nb
                    ij=barras{i}{1};
                    Li=norm(nodos{i+1}{1}-nodos{i}{1});           
                    vEi=material{barras{i}{2}(1)}{1};
                    Izi=perfil{barras{i}{2}(2)}{1};
                    Ma=M(ij(1)); Mb=M(ij(2));
                    ma=mi(j,ij(1)); mb=mi(j,ij(2));
                    p0=barras{i}{3};
                    v(j)=v(j)+Li/(6*vEi*Izi)*(ma*(2*Ma+Mb)+mb*(Ma+2*Mb)+p0*Li^2*(ma+mb)/4);                
                end
                for i=1:NPR                    
                    v(j)=v(j)+mi(j,i)*q(i);
                end
            end                                                                  
        end    
    end
end
